/*
 * Set up the GDT
*/
#include "asm.h"
#include "system.h"
#include "gdt.h"

/* 
 * Install the GDT at 0x90000 and load the GDTR.
 * Flush the segment registers after to load from new GDT.
 * Will occupy 0x90000 -> 0x90038
*/
void k_install_gdt(void)
{
	// GDT starts at this address
	struct GDT * gdt_dest = (struct GDT*) GDT_ADDR;
	// lgdt will load this structure
	struct GDT_PTR gdtr = {GDT_LIMIT, GDT_ADDR};
	
	// Null
	gdt_entry_set_values(0,0,0,0, gdt_dest++);				

	// Ring0 CS
	// Access = 10011010, Gran = 11001111
	gdt_entry_set_values(0, 0xFFFFFFFF, 0x9A, 0xCF, gdt_dest++);

	// Ring0 DS
	// Access = 10010010, Gran = 11001111
	gdt_entry_set_values(0, 0xFFFFFFFF, 0x92, 0xCF, gdt_dest++);

	// Ring3 CS
	// Access = 11111010, Gran = 11001111
	gdt_entry_set_values(0, 0xFFFFFFFF, 0xFA, 0xCF, gdt_dest++);
	// Ring3 DS
	// Access = 11110010, Gran = 11001111
	gdt_entry_set_values(0, 0xFFFFFFFF, 0xF2, 0xCF, gdt_dest++);

	// TSS
	// Access = 10001001; Gran = 11001111;
	gdt_entry_set_values(TSS_ADDR, TSS_LIMIT, 0x89, 0xCF, gdt_dest++);	

	// Extra
	gdt_entry_set_values(0, 0xFFFFFFFF, 0x92, 0xCF, gdt_dest);

	set_gdt_loc(&gdtr);			// Call lgdt ASM macro

	FLUSH_SEGMENTS(0x10);
}


// Set the following values into the gdt entry
void gdt_entry_set_values(	unsigned long base, 
				unsigned long limit, 
				unsigned char access, 
				unsigned char gran, 
				struct GDT* e)
{
	// Set up the descriptor base address
	e->base_low = (base & 0xFFFF);		// First 16 Bits of Base
	e->base_middle = (base >> 16) & 0xFF;	// Next 8 Bits of Base
	e->base_high = (base >> 24) & 0xFF;	// Last 8 Bits of Base

	// Set up descriptor limit
	e->limit_low = (limit & 0xFFFF);
	e->granularity = ((limit >> 16) & 0x0F);

	// Set up granularity and access flags
	e->granularity |= (gran & 0xF0);
	e->access = access;
}


